
-- return the window size
getWindowSize = getWindowSize or function()
	return 714, 315
end

-- create and lay out all the controls
createUI = createUI or function()

	createUICommon()

	-- constants
	local kKnobSpace = 45
	local kKnobSpaceV = 70
	local kEnvW = 120
	local kEnvH = 70
	
	local function makeEnvGroup( id )
		local g = group( "Env " .. id )
		
		local kKnobSpace = 60
		
		local y = 2*kKnobSpaceV + kTextFieldH + 2
		smallKnobAndTextField( g, "Env " .. id .. " Time A1", "Time A1", 0*kKnobSpace, y )
		smallKnobAndTextField( g, "Env " .. id .. " Time A2", "Time A2", 1*kKnobSpace, y )
		smallKnobAndTextField( g, "Env " .. id .. " Time A3", "Time A3", 2*kKnobSpace, y )
		smallKnobAndTextField( g, "Env " .. id .. " Time R1", "Time R1", 3*kKnobSpace, y )
		smallKnobAndTextField( g, "Env " .. id .. " Time R2", "Time R2", 4*kKnobSpace, y )

		smallKnobAndTextField( g, "Env " .. id .. " Level A1", "Level A1", 0.5*kKnobSpace, 0*kKnobSpaceV )
		smallKnobAndTextField( g, "Env " .. id .. " Level A2", "Level A2", 1.5*kKnobSpace, 0*kKnobSpaceV )
		smallKnobAndTextField( g, "Env " .. id .. " Sustain", "Sustain", 2.5*kKnobSpace, 0*kKnobSpaceV )
		smallKnobAndTextField( g, "Env " .. id .. " Level R1", "Level R1", 3.5*kKnobSpace, 0*kKnobSpaceV )

		smallKnobAndTextField( g, "Env " .. id .. " Shape A1", "Shape A1", 0*kKnobSpace, 1*kKnobSpaceV )
		smallKnobAndTextField( g, "Env " .. id .. " Shape A2", "Shape A2", 1*kKnobSpace, 1*kKnobSpaceV )
		smallKnobAndTextField( g, "Env " .. id .. " Shape A3", "Shape A3", 2*kKnobSpace, 1*kKnobSpaceV )
		smallKnobAndTextField( g, "Env " .. id .. " Shape R1", "Shape R1", 3*kKnobSpace, 1*kKnobSpaceV )
		smallKnobAndTextField( g, "Env " .. id .. " Shape R2", "Shape R2", 4*kKnobSpace, 1*kKnobSpaceV )

		local y = kKnobSpaceV + kKnobSize + kTextFieldH + 2
		local h = kTextFieldH + 2
		g:add( paramButton( "Env " .. id .. " Type A1",	"Exp", 0*kKnobSpace, y, kKnobSize, h ) )
		g:add( paramButton( "Env " .. id .. " Type A2",	"Exp", 1*kKnobSpace, y, kKnobSize, h ) )
		g:add( paramButton( "Env " .. id .. " Type A3",	"Exp", 2*kKnobSpace, y, kKnobSize, h ) )
		g:add( paramButton( "Env " .. id .. " Type R1",	"Exp", 3*kKnobSpace, y, kKnobSize, h ) )
		g:add( paramButton( "Env " .. id .. " Type R2",	"Exp", 4*kKnobSpace, y, kKnobSize, h ) )

		knobAndTextField( g, "Env " .. id .. " Velocity Depth", "Velocity", 5*kKnobSpace, 1.5*kKnobSpaceV, kKnobSize, kTextFieldH, 45 )
		g:add( paramButton( "Env " .. id .. " Reset To Zero",		 "RTZ",	5*kKnobSpace, 0.5*kKnobSpaceV, kKnobSpace-pad, kKnobSize ) )
		labelledDropDown( g, "Env " .. id .. " Multiplier",		"Mult",		5*kKnobSpace, 0, kKnobSpace-pad )
	
		return g
	end

	-- make groups and knobs
	--
	
	local x = 0

	theKnobRenderer = smallKnobRenderer

	miscGroup = group( "Setup" )
	labelledDropDown( miscGroup, "Output Mode", "Mode", 0, 0 )
	smallKnobAndTextField( miscGroup, "Trigger Note",	"Note",		kDropdownW+5+0*kKnobSpace, 0 )
	smallKnobAndTextField( miscGroup, "Smoothing",		"Smooth",	kDropdownW+5+1*kKnobSpace, 0 )
	miscGroup:add( paramButton( "Any Note",	"Any Note",		0, kTextFieldH+2*pad, kDropdownW, kTextFieldH ) )

	oscGroup = group( "OSC" )
	smallKnobAndTextField( oscGroup, "OSC Port Offset",	"Port",		0*kKnobSpace, 0 )
	oscGroup:setVisible( false )

	triggerGroup = group( "Trigger" )
	smallKnobAndTextField( triggerGroup, "Trigger Length",	"Length",	0*kKnobSpace, 0 )

	local kVoltageTextW = 45
	voltageGroup = group( "Voltages" )
	x = 0
	knobAndTextField( voltageGroup, "Min Voltage",	"Min",		x, 0*kKnobSpaceV, kKnobSize, kTextFieldH, kVoltageTextW )
	x = x + kVoltageTextW + pad
	knobAndTextField( voltageGroup, "Max Voltage",	"Max",		x, 0*kKnobSpaceV, kKnobSize, kTextFieldH, kVoltageTextW )

	overrideGroup = group( "Overrides" )
	overrideGroup:add( paramButton( 		"Override Gate",	"Gate",		0*kKnobSpace, 0, kKnobSpace-pad, kKnobSize ) )
	smallKnobAndTextField( overrideGroup, 	"Gate",				"Gate",		1*kKnobSpace, 0 )

	local kTextW = 40
	cvTriggerGroup = group( "CV Trigger" )
	x = 0
	cvTriggerGroup:add( paramButton( 		"CV Trigger Enable",	"Enable",		x, 0, 50, kKnobSize ) )
	x = x + 50 + pad
	smallKnobAndTextField( cvTriggerGroup, 	"CV Trigger Threshold",	"Threshold",	x, 0, kTextW )

	env1Group = makeEnvGroup( "1" )

	-- positions!

	local xbase = windowEdgePad
	local ybase = windowEdgePad
	local x = xbase
	local y = ybase
	
	triggerGroup:setPosition( x, y, true )
	x = x + triggerGroup:width() + pad

	overrideGroup:setPosition( x, y, true )
	x = x + overrideGroup:width() + pad

	voltageGroup:setPosition( x, y, true )
	x = x + voltageGroup:width() + pad
	
	oscGroup:setPosition( x, y, true )
	x = x + oscGroup:width() + pad

	y = y + oscGroup:height() + pad

	x = xbase
	
	local envDispX = x
	local envDispY = y

	y = y + voltageGroup:height() + pad
	
	miscGroup:setPosition( x, y, true )
	x = x + miscGroup:width() + pad

	cvTriggerGroup:setPosition( x, y, true )
	x = x + cvTriggerGroup:width() + pad
		
	local envWidth = x - xbase - pad

	x = envDispX + envWidth + pad
	y = ybase
	env1Group:setPosition( x, y, true )
	x = x + env1Group:width() + pad

	-- add directly positioned objects
	--

	x = envDispX
	y = envDispY

	local env = envelopeDisplay( 0, x, y, envWidth, voltageGroup:height() )
	setEnvelopeDisplayColours( env )
	add( env )

end		-- of createUI()
